/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   UART Driver                                                      */
/*  EventID:   Insert Event-ID Test Point                                       */
/*  AppVer :   DT+ Ver1.00`                                                    */
/*  FileID :   37a                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>


/* ========== Parameter ========== */
#define		DT_UINT32				unsigned int	/* Test Point argument type */
#define		DT_VARIABLE_BIT			0x02			/* Variable Test Point */ 
#define		DT_EVTTRG_BIT			0x08			/* EventTrigger */ 
/* ============================================ */


#define STX					0x02
#define ETX					0x03
#define CMD_TESTPOINT		0x07
#define CMD_TESTVALUE		0x08
#define DT_MAX_VAL_SIZE		256

/*===============================================================================
	Func	: _TP_BusOut
	Desc	: Called by Test Point
===============================================================================*/
void _TP_BusOut( DT_UINT32 addr, DT_UINT32 dat )
{
	int	i;
	unsigned char TmpBuf[9];

	TmpBuf[0] = STX;						// STX
	TmpBuf[1] = CMD_TESTPOINT;				// RS-232C Test Point COMMAND
	TmpBuf[2] = (dat >> 8)	& 0x00FF;		// D1
	TmpBuf[3] = (dat)		& 0x00FF;		// D0
	TmpBuf[4] = (addr >>16)	& 0x00FF;		// A2
	TmpBuf[5] = (addr >> 8)	& 0x00FF;		// A1
	TmpBuf[6] = (addr)		& 0x00FF;		// A0
	TmpBuf[7] = CMD_TESTPOINT;				// Check-SUM
	for( i = 2; i < 7; i++) {
		TmpBuf[7] ^= TmpBuf[i];
	}
	TmpBuf[8] = ETX;					// ETX
	fwrite( TmpBuf, 1, 9, stdout ); 
}

/*===============================================================================
	Func	: _TP_MemoryOutput
	Desc	: Called by Variable Test Point
===============================================================================*/
void _TP_MemoryOutput( DT_UINT32 addr, DT_UINT32 dat, void *value, DT_UINT32 size)
{
	unsigned char TmpBuf[DT_MAX_VAL_SIZE+10], csum;
	DT_UINT32 i;
	unsigned char *p = value;

	if( size >= DT_MAX_VAL_SIZE ) size = DT_MAX_VAL_SIZE;
	TmpBuf[0] = STX;					// STX
	TmpBuf[1] = CMD_TESTVALUE;			// RS-232C Test Point COMMAND
	TmpBuf[2] = size;					// Number of Bytes
	TmpBuf[3] = (dat >> 8) & 0x00FF;	// D1
	TmpBuf[4] = (dat)      & 0x00FF;	// D0
	TmpBuf[5] = (addr >>16) & 0x00FF;	// A2
	TmpBuf[6] = (addr >> 8) & 0x00FF;	// A1
	TmpBuf[7] = (addr     ) & 0x00FF;	// A0
	for( i = 0; i < size; i++, p++) {				// Memory Value
		TmpBuf[8+i] = *p;
	}
	csum = TmpBuf[1];
	p = &TmpBuf[2];
	for( i = 2; i < (8+size); i++, p++ ){
		csum ^= *p;
	}
	TmpBuf[i] = csum;
	TmpBuf[i+1] = ETX;
	fwrite( TmpBuf, 1, 10+size, stdout ); 
}

/*===============================================================================
	Func	: _TP_EventTrigger
	Desc	: Called by Event Trigger Test Point
===============================================================================*/
void _TP_EventTrigger( DT_UINT32 addr, DT_UINT32 dat)
{
	_TP_BusOut( addr | DT_EVTTRG_BIT, ((unsigned short)(dat & 0x0FFF)) << 4 );
}

